from AMAT.planet import Planet
from AMAT.vehicle import Vehicle

import numpy as np
import matplotlib.pyplot as plt

planet = Planet('URANUS')
planet.loadAtmosphereModel('../../../atmdata/Uranus/uranus-gram-avg.dat', 0 , 1 ,2, 3, heightInKmFlag=True)
planet.h_skip = 1000.0E3
planet.h_low  = 120e3
planet.h_trap = 100e3

vehicle=Vehicle('Titania', 3200.0, 146 , 0.24, np.pi*4.5**2.0, 0.0, 1.125, planet)
vehicle.setInitialState(1000.0,-15.22,75.55,29.2877,88.687,-11.7 ,0.0,0.0)
vehicle.setSolverParams(1E-6)
# Set the guidance parameters described in the paper.
# See the function description for parameter details.

# Set max roll rate constraint to 30 deg/s
vehicle.setMaxRollRate(30.0)

# Set Ghdot = 83
# Set Gq = 9.0
# Set v_switch_kms = 16.0 :: this parameter is not really used, and should just be set to low value
# Set low_Alt_km = 120
# Set numPoints_lowAlt = 101
# Set hdot_threshold = -500 m/s
vehicle.setEquilibriumGlideParams(83.0, 9.0, 16.0, 120.0, 101, -500.0)

# Set target orbit parameters
# periapsis = 4000.0 km
# apoapsis = 550,000 km :: set slightly above target (500e3) to account for bias in targeting algorithm
# apoapsis tolerance = 10 km
vehicle.setTargetOrbitParams(4000.0, 550.0E3, 10.0E3)

# Set path to atmfiles with randomly perturbed atmosphere files.
atmfiles = ['../../../atmdata/Uranus/perturbed/G-01.txt',
            '../../../atmdata/Uranus/perturbed/G-02.txt',
            '../../../atmdata/Uranus/perturbed/G-03.txt',
            '../../../atmdata/Uranus/perturbed/G-04.txt',
            '../../../atmdata/Uranus/perturbed/G-05.txt']


vehicle.setupMonteCarloSimulation(1201, 200, atmfiles, 0, 1, 2, 3, 4, True, \
                                 -11.70, 0.10, 0.24, 0.013, 0.5, 0.1, 2400.0)

vehicle.runMonteCarlo2(2000, '../../../data/acta-astronautica/uranus-orbiter-probe/MCB')


"""
test_launcher.py
Basic tests for AMAT.launcher

"""

import unittest

try:
	from AMAT.launcher import Launcher
except ModuleNotFoundError:
	raise ModuleNotFoundError("Cannot import Launcher from AMAT.Launcher")


class TestLauncher(unittest.TestCase):

	def test_falcon_heavy_expendable(self):
		launcher = Launcher('Falcon Heavy Expendable', datafile='launcher-data/falcon-heavy-expendable.csv')
		# Compare with numbers from https://elvperf.ksc.nasa.gov/Pages/Query.aspx
		self.assertAlmostEqual(launcher.launchMass(29), 8405 , delta=50)

	def test_falcon_heavy_recovery(self):
		launcher = Launcher('Falcon Heavy Reusable', datafile='launcher-data/falcon-heavy-reusable.csv')
		# Compare with numbers from https://elvperf.ksc.nasa.gov/Pages/Query.aspx
		self.assertAlmostEqual(launcher.launchMass(29), 2850 , delta=50)

	def test_atlas_V_401(self):
		launcher = Launcher('Atlas V401', datafile='launcher-data/atlas-v401.csv')
		self.assertAlmostEqual(launcher.launchMass(11.94), 2316 , delta=50)

	def test_atlas_V_551(self):
		launcher = Launcher('Atlas V551', datafile='launcher-data/atlas-v551.csv')
		self.assertAlmostEqual(launcher.launchMass(11.94), 4879 , delta=50)

	def test_delta_IV_Heavy(self):
		launcher = Launcher('Delta IVH', datafile='launcher-data/delta-IVH.csv')
		self.assertAlmostEqual(launcher.launchMass(11.94), 8153 , delta=50)

	def test_falcon_heavy_expendable_with_kick1(self):
		launcher = Launcher('Falcon Heavy Expendable with Kick', datafile='launcher-data/falcon-heavy-expendable-w-star-48.csv')
		self.assertAlmostEqual(launcher.launchMass(29.36), 7800 , delta=100)

	def test_falcon_heavy_expendable_with_kick2(self):
		launcher = Launcher('Falcon Heavy Expendable with Kick', datafile='launcher-data/falcon-heavy-expendable-w-star-48.csv')
		self.assertAlmostEqual(launcher.launchMass(80), 3010 , delta=200)

	def test_vulcan_6(self):
		launcher = Launcher('Vulcan-6', datafile='launcher-data/vulcan-centaur-w-6-solids.csv')
		self.assertAlmostEqual(launcher.launchMass(29), 6328, delta=50)

	def test_vulcan_6_with_kick(self):
		launcher = Launcher('Vulcan-6-with-kick', datafile='launcher-data/vulcan-centaur-w-6-solids-w-star-48.csv')
		self.assertAlmostEqual(launcher.launchMass(80), 2586, delta=200)

	def test_sls_block_1(self):
		launcher = Launcher('SLS-Block-1', datafile='launcher-data/sls-block-1.csv')
		self.assertAlmostEqual(launcher.launchMass(80), 5400, delta=100)

	def test_sls_block_1B(self):
		launcher = Launcher('SLS-Block-1B', datafile='launcher-data/sls-block-1B.csv')
		self.assertAlmostEqual(launcher.launchMass(80), 8500, delta=100)

	def test_sls_block_1B_kick(self):
		launcher = Launcher('SLS-Block-1B-with-kick', datafile='launcher-data/sls-block-1B-with-kick.csv')
		self.assertAlmostEqual(launcher.launchMass(80), 10400, delta=100)

	def test_sls_block_1B_kick_2(self):
		launcher = Launcher(launcherID='sls-block-1B-with-kick',
							datafile='launcher-data/sls-block-1B-with-kick.csv')
		self.assertAlmostEqual(launcher.launchMass(111.0), 6600, delta=100)


if __name__ == '__main__':
	unittest.main()













# SOURCE FILENAME : telecom.py
# AUTHOR          : Athul Pradeepkumar Girija, athulpg007@gmail.com
# DATE CREATED    : 11/12/2022, 08:42 MT
# DATE MODIFIED   : 11/12/2022 08:42 MT
# REMARKS         : Compute the link margin and data rate for a telecom link.

import numpy as np
from scipy.integrate import cumulative_trapezoid


class Link:
	"""
	Computes the link budget for a telecom link.

	Attributes
	-----------
	freq : float
		Frequency, Hz
	wavelength : float
		Wavelength, m
	Pt : float
		Transmitter power, W
	Pt_dB : float
		Transmitter power, dBW
	Lt_dB : float
		Transmitter antenna loss, dB
	L_sc_dB : float
		Spacecraft circuit loss, dB
	Dt : float
		Transmitter antenna diameter, m
	At : float
		Transmitter antenna area, m2
	eta_t : float
		Transmitter antenna efficiency
	Gt_dBi : float
		Transmitter antenna gain, dBi
		Computed using Dt and eta_t if provided, or from Gt_dBi
	Dr : float
		Receiver antenna diameter, m
	Ar : float
		Receiver antenna area, m2
	eta_r : float
		Receiver antenna efficiency
	Gr_dBi: float
		Receiver gain, dBi
		Computed using Dr and eta_r if provided, or from Gr_dBi
	Ts : float
		System Noise Temperature, K
	Ts_dBK : float
		System Noise Temperature, dBK
	range : float
		Link distance, m
	Ls_dB : float
		Free space loss, dB
	La_dB : float
		Atmospheric attenuation, dB
	L_other_dB : float
		Other losses, dB
	L_DSN_dB : float
		DSN System Loss, dB
	Ka_dB : float
		Boltzmann's constant term
	R : float
		Data rate, bps
	R_dBHz : float
		Data rate, dBHz
	EB_N0_req : float
		Required Eb/N0
	EB_N0 : float
		Required Eb/N0
	EB_N0_margin : float
		Eb/N0 margin
	"""

	def __init__(self, freq, Pt, Ts, range_km, L_other_dB, rate_kbps, Eb_N0_req,
				 Dt=None, eta_t=None, Gt_dBi=None, Dr=None, eta_r=None, Gr_dBi=None,
				 Lt_dB=1.0, L_sc_dB=0.2, La_dB=0.35, L_DSN_dB=1.0):
		"""

		Parameters
		----------
		freq : float
			Frequency, GHz
		Pt : float
			Transmitter power, W
		Ts : float
			System Noise Temperature, K
		range_km : float
			link distance, km
		L_other_dB : float
			other losses, dB
		rate_kbps : float
			data rate, kbps
		Eb_N0_req : float
			required Eb/No
		Dt : float
			Transmitter antenna diameter, m
		eta_t : float
			Transmitter antenna efficiency
		Gt_dBi : float, optional
			Transmitter antenna gain, dBi
			Must be provided if Dt and eta_t are not provided.
		Dr : float
			Receiver antenna diameter, m
		eta_r : float
			Receiver antenna efficiency
		Gr_dBi: float
			Receiver gain, dBi
			Must be provided if Dr and eta_r are not provided.
		Lt_dB : float, optional
			Transmitter antenna loss, dB; defaults to 1.0 dB
		L_sc_dB : float, optional
			Spacecraft circuit loss, dB; defaults to 0.2 dB
		La_dB : float, optional
			atmospheric attenuation loss, dB; defaults to 0.35 dB
		L_DSN_dB : float, optional
			DSN system losses, dBl defaults to 1.0 dB

		"""

		self.freq = freq*1E9
		self.wavelength = 3E8/self.freq
		self.Pt = Pt
		self.Pt_dB = self.compute_dB(self.Pt)
		self.Lt_dB = Lt_dB
		self.L_sc_dB = L_sc_dB

		# if Dt and eta_t are provided, compute transmitter gain using these values,
		# else if Gt_dBi is provided, use this value directly for transmitter gain
		if Dt and eta_t:
			self.Dt = Dt
			self.At = 0.25*np.pi*self.Dt*self.Dt
			self.eta_t = eta_t
			self.Gt = self.compute_antenna_gain(self.eta_t, self.Dt, self.wavelength)
			self.Gt_dBi = self.compute_dB(self.Gt)
		elif Gt_dBi:
			self.Dt = None
			self.At = None
			self.eta_t = None
			self.Gt_dBi = Gt_dBi

		# if Dr and eta_r are provided, compute receiver gain using these values,
		# else if Gr_dBi is provided, use this value directly for receiver gain
		if Dr and eta_r:
			self.Dr = Dt
			self.Ar = 0.25 * np.pi * self.Dr * self.Dr
			self.eta_r = eta_r
			self.Gr = self.compute_antenna_gain(self.eta_r, self.Dr, self.wavelength)
			self.Gr_dBi = self.compute_dB(self.Gr)
		elif Gr_dBi:
			self.Dr = None
			self.Ar = None
			self.eta_r = None
			self.Gr_dBi = Gr_dBi

		self.Ts = Ts
		self.Ts_dBK = self.compute_dB(self.Ts)
		self.range = range_km*1E3
		self.Ls_dB = self.compute_free_space_loss_dB(self.range, self.wavelength)
		self.La_dB = La_dB
		self.L_other_dB = L_other_dB
		self.L_DSN_dB = L_DSN_dB
		self.Ka = 1.38E-23
		self.Ka_dB = self.compute_dB(self.Ka)
		self.R = rate_kbps*1E3
		self.R_dBHz = self.compute_dB(self.R)
		self.EB_N0_req = Eb_N0_req

		self.EB_N0 = self.Pt_dB + self.Gt_dBi - self.Lt_dB - self.L_sc_dB + self.Gr_dBi - self.Ts_dBK -\
					 self.Ls_dB - self.La_dB - self.L_other_dB - self.L_DSN_dB - self.R_dBHz - self.Ka_dB
		self.EB_N0_margin = self.EB_N0 - self.EB_N0_req

	def compute_dB(self, X):
		"""

		Parameters
		----------
		X : float
			any parameter to be converted to dB

		Returns
		-------
		ans : float
			Power in dBW

		"""
		return 10*np.log10(X)

	def compute_antenna_gain(self, eta, D, wavelength):
		"""

		Parameters
		----------
		eta : float
			antenna efficiency
		D : float
			antenna diameter, m
		wavelength : float
			wavelength, m

		Returns
		-------
		ans : float
			antenna gain, before converting to dBi
		"""
		return eta*((np.pi*D)/wavelength)**2

	def compute_free_space_loss_dB(self, range, wavelength):
		"""

		Parameters
		----------
		range : float
			link distance, m
		wavelength : float
			wavelength, m

		Returns
		-------
		ans : float
			free space loss, dB
		"""
		return 20*np.log10(4*np.pi*range/wavelength)

	def print_link_budget(self):
		print("\n")
		print("-------------------------------------------")
		print(f"Frequency, GHz:          {self.freq/1E9}")
		print(f"Transmitter Power, W:    {self.Pt}")
		print(f"Transmitter Power, dB:   {round(self.Pt_dB,2)}")
		print(f"Transmitter Loss, dB:   -{round(self.Lt_dB, 2)}")
		print(f"S/C Circuit Loss, dB:   -{round(self.L_sc_dB, 2)}")
		print(f"Transmit antenna D., m:  {self.Dt}")
		print(f"Transmit antenna eff:    {self.eta_t}")
		print(f"Transmitter Gain, dBi:   {round(self.Gt_dBi,2)}")
		print(f"Receive antenna D., m:   {self.Dr}")
		print(f"Receive antenna eff:     {self.eta_r}")
		print(f"Receiver Gain, dBi:      {round(self.Gr_dBi,2)}")
		print(f"System Noise Temp, K:    {self.Ts}")
		print(f"System Noise Temp, dBK: -{round(self.Ts_dBK,2)}")
		print(f"Link Distance, km:       {self.range/1E3}")
		print(f"Free Space Loss, dB:    -{round(self.Ls_dB,2)}")
		print(f"Atmospheric Loss, dB:   -{self.La_dB}")
		print(f"Other Losses, dB:       -{self.L_other_dB}")
		print(f"Boltzmann's const, dB:  +{round(-self.Ka_dB,2)}")
		print(f"Data rate, kbps:         {self.R/1E3}")
		print(f"Data rate, dBhz:        -{round(self.R_dBHz,2)}")
		print("-------------------------------------------")
		print(f"Available Eb/N0:         {round(self.EB_N0,2)}")
		print(f"Required Eb/N0:          {self.EB_N0_req}")
		print(f"Eb/N0 Margin:            {round(self.EB_N0_margin,2)}")
		print("-------------------------------------------")

	def compute_data_volume(self, visibility, schedule):
		"""

		Parameters
		----------
		visibility : one of AMAT.visibility.LanderToPlanet, AMAT.visibility.LanderToOrbiter,
					 AMAT.visibility.OrbiterToPlanet objects
		schedule : AMAT.telecom.Schedule
			Transmit schedule

		Returns
		-------
		"""

		self.visibility = visibility
		self.schedule = schedule
		self.t_array = self.visibility.t_array
		self.data_volume_array = cumulative_trapezoid((self.R)*self.visibility.visible_array*self.schedule.transmit_array,
										  self.t_array, initial=0)


class Schedule:
	def __init__(self, t_seconds, num_points, t_transmit_times_list):
		"""

		Parameters
		----------
		t_seconds : float
			time for schedule in seconds
		num_points : int
			number of points to compute for the schedule
		t_transmit_times_list : list of numpy.ndarray
			Specify the times the data link is transmitting
			eg: [np.array([0, 86400]), np.array([123000, 125000])]
		"""
		self.t_array = np.linspace(0, t_seconds, num_points)
		self.transmit_array = np.zeros(num_points, dtype=bool)

		for i, t in enumerate(self.t_array):
			for window in t_transmit_times_list:
				if window[0] <= t <= window[1]:
					self.transmit_array[i] = True
					break
				else:
					self.transmit_array[i] = False



"""
test_planet.py

Tests to check if Planet objects can be created and loaded with
atmospheric profiles.

"""

import unittest


try:
	from AMAT.planet import Planet
except ModuleNotFoundError:
	raise ModuleNotFoundError("Cannot import Planet from AMAT.planet")


class TestCreatePlanet(unittest.TestCase):

	def test_create_planet_venus(self):
		"""
		Test that if a Planet object instances can be created for
		various Solar System destinations
		"""
		planet1 = Planet("VENUS")
		message = "Could not create Planet instance for Venus"
		self.assertIsInstance(planet1, Planet, message)
		self.assertEqual(planet1.ID, "VENUS")

	def test_create_planet_earth(self):
		planet2 = Planet("EARTH")
		message = "Could not create Planet instance for Earth"
		self.assertIsInstance(planet2, Planet, message)
		self.assertEqual(planet2.ID, "EARTH")

	def test_create_planet_mars(self):
		planet3 = Planet("MARS")
		message = "Could not create Planet instance for Mars"
		self.assertIsInstance(planet3, Planet, message)
		self.assertEqual(planet3.ID, "MARS")

	def test_create_planet_jupiter(self):
		planet4 = Planet("JUPITER")
		message = "Could not create Planet instance for Jupiter"
		self.assertIsInstance(planet4, Planet, message)
		self.assertEqual(planet4.ID, "JUPITER")

	def test_create_planet_saturn(self):
		planet5 = Planet("SATURN")
		message = "Could not create Planet instance for Saturn"
		self.assertIsInstance(planet5, Planet, message)
		self.assertEqual(planet5.ID, "SATURN")

	def test_create_planet_titan(self):
		planet6 = Planet("TITAN")
		message = "Could not create Planet instance for Titan"
		self.assertIsInstance(planet6, Planet, message)
		self.assertEqual(planet6.ID, "TITAN")

	def test_create_planet_uranus(self):
		planet7 = Planet("URANUS")
		message = "Could not create Planet instance for Uranus"
		self.assertIsInstance(planet7, Planet, message)
		self.assertEqual(planet7.ID, "URANUS")

	def test_create_planet_neptune(self):
		planet8 = Planet("NEPTUNE")
		message = "Could not create Planet instance for Neptune"
		self.assertIsInstance(planet8, Planet, message)
		self.assertEqual(planet8.ID, "NEPTUNE")

	def test_create_planet_invalid(self):
		try:
			planet9 = Planet("XXXXXXX")
			assert planet9.ID is None
		except ValueError:
			assert True


class TestPlanetConstants(unittest.TestCase):

	def test_planet_GM(self):
		"""
		check GM values for all planet objects
		"""
		planet1 = Planet("VENUS")
		planet2 = Planet("EARTH")
		planet3 = Planet("MARS")
		planet4 = Planet("JUPITER")
		planet5 = Planet("SATURN")
		planet6 = Planet("TITAN")
		planet7 = Planet("URANUS")
		planet8 = Planet("NEPTUNE")

		self.assertAlmostEqual(planet1.GM, 3.248599E14, places=4, msg="Check Venus GM")
		self.assertAlmostEqual(planet2.GM, 3.986004E14, places=4, msg="Check Earth GM")
		self.assertAlmostEqual(planet3.GM, 4.282837E13, places=4, msg="Check Mars GM")
		self.assertAlmostEqual(planet4.GM, 1.26686534E17, places=4, msg="Check Jupiter GM")
		self.assertAlmostEqual(planet5.GM, 3.7931187E16, places=4, msg="Check Saturn GM")
		self.assertAlmostEqual(planet6.GM, 8.9780000E12, places=4, msg="Check Titan GM")
		self.assertAlmostEqual(planet7.GM, 5.793939E15,  places=4, msg="Check Uranus GM")
		self.assertAlmostEqual(planet8.GM, 6.8365299E15, places=4, msg="Check Neptune GM")

	def test_planet_RP(self):
		"""
		check GM values for all planet objects
		"""
		planet1 = Planet("VENUS")
		planet2 = Planet("EARTH")
		planet3 = Planet("MARS")
		planet4 = Planet("JUPITER")
		planet5 = Planet("SATURN")
		planet6 = Planet("TITAN")
		planet7 = Planet("URANUS")
		planet8 = Planet("NEPTUNE")

		self.assertAlmostEqual(planet1.RP, 6051.8000E3, delta=10e3, msg="Check Venus RP")
		self.assertAlmostEqual(planet2.RP, 6371.0000E3, delta=10e3, msg="Check Earth RP")
		self.assertAlmostEqual(planet3.RP, 3389.5000E3, delta=10e3, msg="Check Mars RP")
		self.assertAlmostEqual(planet4.RP, 69911.0E3, delta=100e3, msg="Check Jupiter RP")
		self.assertAlmostEqual(planet5.RP, 58232.0E3, delta=100e3, msg="Check Saturn RP")
		self.assertAlmostEqual(planet6.RP, 2575.0000E3, delta=10e3, msg="Check Titan RP")
		self.assertAlmostEqual(planet7.RP, 25559.0E3, delta=100e3, msg="Check Uranus RP")
		self.assertAlmostEqual(planet8.RP, 24622.000E3, delta=100e3, msg="Check Neptune RP")

	def test_planet_h_thres_h_trap(self):

		"""
		check h_thres and h_trap for all planets
		"""
		planet1 = Planet("VENUS")
		planet2 = Planet("EARTH")
		planet3 = Planet("MARS")
		planet4 = Planet("JUPITER")
		planet5 = Planet("SATURN")
		planet6 = Planet("TITAN")
		planet7 = Planet("URANUS")
		planet8 = Planet("NEPTUNE")

		"""
		check h_thres is set to a sufficiently high value for all planets
		"""
		self.assertGreaterEqual(planet1.h_thres, 100.0E3, msg="Venus atm. cut off height likely too low.")
		self.assertGreaterEqual(planet2.h_thres, 100.0E3, msg="Earth atm. cut off height likely too low.")
		self.assertGreaterEqual(planet3.h_thres, 100.0E3, msg="Mars atm. cut off height likely too low.")
		self.assertGreaterEqual(planet4.h_thres, 800.0E3, msg="Jupiter atm. cut off height likely too low.")
		self.assertGreaterEqual(planet5.h_thres, 800.0E3, msg="Saturn atm. cut off height likely too low.")
		self.assertGreaterEqual(planet6.h_thres, 600.0E3, msg="Titan atm. cut off height likely too low.")
		self.assertGreaterEqual(planet7.h_thres, 800.0E3, msg="Uranus atm. cut off height likely too low.")
		self.assertGreaterEqual(planet8.h_thres, 800.0E3, msg="Neptune atm. cut off height likely too low.")

		"""
		check h_thres (atm. cut off) and h_skip agree to within 1 km.
		"""
		self.assertAlmostEqual(planet1.h_thres, planet1.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Venus")
		self.assertAlmostEqual(planet2.h_thres, planet2.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Earth")
		self.assertAlmostEqual(planet3.h_thres, planet3.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Mars")
		self.assertAlmostEqual(planet4.h_thres, planet4.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Jupiter")
		self.assertAlmostEqual(planet5.h_thres, planet5.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Saturn")
		self.assertAlmostEqual(planet6.h_thres, planet6.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Titan")
		self.assertAlmostEqual(planet7.h_thres, planet7.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Uranus")
		self.assertAlmostEqual(planet8.h_thres, planet8.h_skip, delta=1e3, msg="h_thres, h_skip disagree for Neptune")

		"""
		check h_trap (traj. cut-off low-altitude) is set to a sufficiently low value
		"""
		self.assertLessEqual(planet1.h_trap, 40.0E3, msg="Venus trap alt. likely too high.")
		self.assertLessEqual(planet2.h_trap, 40.0E3, msg="Earth trap alt. likely too high.")
		self.assertLessEqual(planet3.h_trap, 60.0E3, msg="Mars trap alt. likely too high.")
		self.assertLessEqual(planet4.h_trap, 100.0E3, msg="Jupiter trap alt. likely too high.")
		self.assertLessEqual(planet5.h_trap, 100.0E3, msg="Saturn trap alt. likely too high.")
		self.assertLessEqual(planet6.h_trap, 100.0E3, msg="Titan trap alt. likely too high.")
		self.assertLessEqual(planet7.h_trap, 100.0E3, msg="Uranus trap alt. likely too high.")
		self.assertLessEqual(planet8.h_trap, 100.0E3, msg="Neptune trap alt. likely too high.")


class TestLoadAtmosphereModels(unittest.TestCase):
	"""
	Test atmosphere models can be loaded for various planetary bodies.
	"""

	def test_load_atm_model_venus(self):
		planet1 = Planet("VENUS")
		try:
			planet1.loadAtmosphereModel('atmdata/Venus/venus-gram-avg.dat', 0, 1, 2, 3)
			assert True
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_earth(self):
		planet2 = Planet("EARTH")
		try:
			planet2.loadAtmosphereModel('atmdata/Earth/earth-gram-avg.dat', 0, 1, 2, 3)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_mars(self):
		planet3 = Planet("MARS")
		try:
			planet3.loadAtmosphereModel('atmdata/Mars/mars-gram-avg.dat', 0, 1, 2, 3)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_jupiter(self):
		planet4 = Planet("JUPITER")
		try:
			planet4.loadAtmosphereModel('atmdata/Jupiter/jupiter-galileo-asi.dat', 0, 1, 2, 3, heightInKmFlag=True)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_saturn(self):
		planet5 = Planet("SATURN")
		try:
			planet5.loadAtmosphereModel('atmdata/Saturn/saturn-nominal.dat', 0, 1, 2, 3, heightInKmFlag=True)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_titan(self):
		planet6 = Planet("TITAN")
		try:
			planet6.loadAtmosphereModel('atmdata/Titan/titan-gram-avg.dat', 0, 1, 2, 3)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_uranus(self):
		planet7 = Planet("URANUS")
		try:
			planet7.loadAtmosphereModel('atmdata/Uranus/uranus-ames.dat', 0, 1, 2, 3)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

	def test_load_atm_model_neptune(self):
		planet8 = Planet("NEPTUNE")
		try:
			planet8.loadAtmosphereModel('atmdata/Neptune/neptune-gram-avg.dat', 0, 7, 6, 5, heightInKmFlag=True)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")


class TestAtmosphereModelFiles(unittest.TestCase):
	def test_num_cols_atm_files(self):
		# check loaded atmosphere data has equal number of rows for 
		# height, temperature, pressure and density

		planet1 = Planet("VENUS")
		planet2 = Planet("EARTH")
		planet3 = Planet("MARS")
		planet4 = Planet("JUPITER")
		planet5 = Planet("SATURN")
		planet6 = Planet("TITAN")
		planet7 = Planet("URANUS")
		planet8 = Planet("NEPTUNE")

		planet1.loadAtmosphereModel('atmdata/Venus/venus-gram-avg.dat', 0, 1, 2, 3)
		planet2.loadAtmosphereModel('atmdata/Earth/earth-gram-avg.dat', 0, 1, 2, 3)
		planet3.loadAtmosphereModel('atmdata/Mars/mars-gram-avg.dat', 0, 1, 2, 3)
		planet4.loadAtmosphereModel('atmdata/Jupiter/jupiter-galileo-asi.dat', 0, 1, 2, 3, heightInKmFlag=True)
		planet5.loadAtmosphereModel('atmdata/Saturn/saturn-nominal.dat', 0, 1, 2, 3, heightInKmFlag=True)
		planet6.loadAtmosphereModel('atmdata/Titan/titan-gram-avg.dat', 0, 1, 2, 3)
		planet7.loadAtmosphereModel('atmdata/Uranus/uranus-ames.dat', 0, 1, 2, 3)
		planet8.loadAtmosphereModel('atmdata/Neptune/neptune-gram-avg.dat', 0, 7, 6, 5, heightInKmFlag=True)

		self.assertEqual(len(planet1.ATM_height), len(planet1.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet1.ATM_height), len(planet1.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet1.ATM_height), len(planet1.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet2.ATM_height), len(planet2.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet2.ATM_height), len(planet2.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet2.ATM_height), len(planet2.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet3.ATM_height), len(planet3.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet3.ATM_height), len(planet3.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet3.ATM_height), len(planet3.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet4.ATM_height), len(planet4.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet4.ATM_height), len(planet4.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet4.ATM_height), len(planet4.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet5.ATM_height), len(planet5.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet5.ATM_height), len(planet5.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet5.ATM_height), len(planet5.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet6.ATM_height), len(planet6.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet6.ATM_height), len(planet6.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet6.ATM_height), len(planet6.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet7.ATM_height), len(planet7.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet7.ATM_height), len(planet7.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet7.ATM_height), len(planet7.ATM_density), msg="Number of rows disagree in atm. data file")

		self.assertEqual(len(planet8.ATM_height), len(planet8.ATM_temp), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet8.ATM_height), len(planet8.ATM_pressure), msg="Number of rows disagree in atm. data file")
		self.assertEqual(len(planet8.ATM_height), len(planet8.ATM_density), msg="Number of rows disagree in atm. data file")

		# check h_thres is set to a lower value (<=) than the maximum height for 
		# which data is available

		self.assertLessEqual(planet1.h_thres, max(planet1.ATM_height), msg="h_thres > max. height from table for Venus.")
		self.assertLessEqual(planet2.h_thres, max(planet2.ATM_height), msg="h_thres > max. height from table for Earth.")
		self.assertLessEqual(planet3.h_thres, max(planet3.ATM_height), msg="h_thres > max. height from table for Mars.")
		self.assertLessEqual(planet4.h_thres, max(planet4.ATM_height), msg="h_thres > max. height from table for Jupiter.")
		self.assertLessEqual(planet5.h_thres, max(planet5.ATM_height), msg="h_thres > max. height from table for Saturn.")
		self.assertLessEqual(planet6.h_thres, max(planet6.ATM_height), msg="h_thres > max. height from table for Titan.")
		self.assertLessEqual(planet7.h_thres, max(planet7.ATM_height), msg="h_thres > max. height from table for Uranus.")
		self.assertLessEqual(planet8.h_thres, max(planet8.ATM_height), msg="h_thres > max. height from table for Neptune.")


class TestAtmosphereValues(unittest.TestCase):

	def test_density_at_surface(self):
		"""
		check some atmospheric data values are correct.
		"""
		planet1 = Planet("VENUS")
		planet2 = Planet("EARTH")
		planet3 = Planet("MARS")
		planet4 = Planet("JUPITER")
		planet5 = Planet("SATURN")
		planet6 = Planet("TITAN")
		planet7 = Planet("URANUS")
		planet8 = Planet("NEPTUNE")

		planet1.loadAtmosphereModel('atmdata/Venus/venus-gram-avg.dat', 0, 1, 2, 3)
		planet2.loadAtmosphereModel('atmdata/Earth/earth-gram-avg.dat', 0, 1, 2, 3)
		planet3.loadAtmosphereModel('atmdata/Mars/mars-gram-avg.dat', 0, 1, 2, 3)
		planet4.loadAtmosphereModel('atmdata/Jupiter/jupiter-galileo-asi.dat', 0, 1, 2, 3, heightInKmFlag=True)
		planet5.loadAtmosphereModel('atmdata/Saturn/saturn-nominal.dat', 0, 1, 2, 3, heightInKmFlag=True)
		planet6.loadAtmosphereModel('atmdata/Titan/titan-gram-avg.dat', 0, 1, 2, 3)
		planet7.loadAtmosphereModel('atmdata/Uranus/uranus-ames.dat', 0, 1, 2, 3)
		planet8.loadAtmosphereModel('atmdata/Neptune/neptune-gram-avg.dat', 0, 7, 6, 5, heightInKmFlag=True)

		self.assertAlmostEqual(planet1.density(h=0.0), 64.79, places=1)
		self.assertAlmostEqual(planet2.density(h=0.0), 1.221, places=1)
		self.assertAlmostEqual(planet3.density(h=0.0), 0.01319, places=3)
		self.assertAlmostEqual(planet4.density(h=0.0), 0.16288, places=3)
		self.assertAlmostEqual(planet5.density(h=0.0), 0.18658, places=3)
		self.assertAlmostEqual(planet6.density(h=0.0), 5.43503, places=2)
		self.assertAlmostEqual(planet7.density(h=0.0), 0.37879, places=2)
		self.assertAlmostEqual(planet8.density(h=0.0), 0.44021, places=2)


if __name__ == '__main__':
	unittest.main()



"""
test_craigLyne2005.py

Compare results from Craig and Lyne [2005, JSR]

"""

import unittest

try:
    from AMAT.planet import Planet
except ModuleNotFoundError:
    raise ModuleNotFoundError("Cannot import Planet from AMAT.planet")

try:
	from AMAT.vehicle import Vehicle
except ModuleNotFoundError:
	raise ModuleNotFoundError("Cannot import Vehicle from AMAT.vehicle")

class CraigLyne2005(unittest.TestCase):

	def test_craig_lyne_2005(self):

		planet1 = Planet("VENUS")

		try:
			planet1.loadAtmosphereModel('atmdata/Venus/venus-gram-avg.dat', 0 , 1 ,2, 3)
		except OSError:
			raise OSError("File not found. Check file path/name, and make sure file is present.")

		# vehicle and entry state description from Craig and Lyne, 
		# Parametric Study of Aerocapture for Missions to Venus, 
		# Journal of Spacecraft and Rockets, 
		# Vol. 42, No. 6., 2005. DOI:10.2514/1.2589
		vehicle1=Vehicle('Apollo', 300.0, 78.0, 0.35, 3.1416, 0.0, 1.54, planet1)
		vehicle1.setInitialState(180.0,0.0,0.0,12.0,0.0,-4.5,0.0,0.0)
		vehicle1.setSolverParams(1E-6)

		print("Computing aerocapture corridor bounds from Craig and Lyne [JSR, 2005]...")
		overShootLimit, exitflag_os  = vehicle1.findOverShootLimit (2400.0, 1.0, -20.0, -4.0, 1E-2, 407.0)
		underShootLimit,exitflag_us  = vehicle1.findUnderShootLimit(2400.0, 1.0, -20.0, -4.0, 1E-2, 407.0)

		# check exitflag_os, exitflag_us = 1 (indicates AMAT found a solution)
		self.assertEqual(exitflag_os, 1)
		self.assertEqual(exitflag_us, 1)

		# compare with values from Fig. 2 Craig and Lyne [JSR, 2005] @ V = 12 km/s
		self.assertAlmostEqual(overShootLimit,  -7.0468, places=2)
		self.assertAlmostEqual(underShootLimit, -9.4396, places=2)


if __name__ == '__main__':
	unittest.main()

		

